package com.aidex.web.controller.app;


import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.aidex.common.app.domain.vo.LocationVO;
import com.aidex.common.app.domain.vo.SysWharfVO;
import com.aidex.common.app.server.IAppService;
import com.aidex.common.core.domain.BaseEntity;
import com.aidex.common.core.domain.R;
import com.aidex.common.core.page.PageDomain;
import com.aidex.common.gps.domain.LocationEntity;
import com.aidex.common.plush.domain.vo.SysShipExtendListResVO;
import com.aidex.common.plush.domain.vo.SysShipExtendListVO;
import com.aidex.common.plush.domain.vo.SysShipExtendVO;
import com.aidex.common.plush.domain.vo.SysShipResVO;
import com.aidex.common.utils.DateUtils;
import com.aidex.common.utils.json.JSONObject;
import com.aidex.common.utils.uuid.IdUtils;
import com.aidex.quartz.task.RefreshAccessTokenTask;
import com.aidex.system.domain.*;
import com.aidex.system.domain.vo.SysReservationDataHistoryVO;
import com.aidex.system.domain.vo.SysRouteVo;
import com.aidex.system.service.*;
import com.alibaba.fastjson2.JSONArray;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.v3.oas.annotations.tags.Tag;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.common.service.WxService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;


@RestController
@RequestMapping("/app")
@Tag(name = "小程序相关接口", description = "小程序管调用接口")
public class AppController {

    protected static final Logger logger = LoggerFactory.getLogger(AppController.class);


    @Autowired
    private IAppService iAppService;

    @Autowired
    private SysWharfService sysWharfService;

    @Autowired
    private SysDictDataService sysDictDataService;

    @Autowired
    private SysNoticeService noticeService;

    @Resource
    private WxMaService wxMaService;

    @Autowired
    private SysShipService sysShipService;

	@GetMapping("/carousel")
    @ApiOperation(value = "获取轮播信息", notes = "获取轮播信息", produces = "application/json")
    public R<List<SysNotice>> carousel()
    {
        return R.data(iAppService.getCarouselList());
    }

    @GetMapping("/notice")
    @ApiOperation(value = "获取通知公告", notes = "获取通知公告", produces = "application/json")
    public R<List<SysNotice>> notice()
    {
        return R.data(iAppService.getNoticeList());
    }

    @GetMapping("/noticeAndCarouseldetails/{id}")
    @ApiOperation(value = "获取轮播或公告详情", notes = "获取轮播或公告详情", produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "数据主键ID", required = true, dataType = "String", paramType = "query")})
    public R<SysNotice> noticeAndCarouseldetails(@PathVariable("id") String id)
    {
        return R.data(iAppService.findCarouselAndNoticeDetails(id));
    }

    @GetMapping("/wharf")
    @ApiOperation(value = "获取全部站点列表", notes = "获取全部站点列表", produces = "application/json")
    public R<List<SysWharf>> findSysWharfList()
    {
        return R.data(iAppService.findSysWharfList());
    }

    @GetMapping("/location")
    @ApiOperation(value = "获取离我最近的船只位置", notes = "获取离我最近的船只位置", produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "direction", value = "正向（TRUE）/反向（FALSE）、默认正向", required = true, dataType = "Boolean"),
            @ApiImplicitParam(name = "lat", value = "纬度", required = true, dataType = "String"),
            @ApiImplicitParam(name = "lon", value = "经度", required = true, dataType = "String")})
    public R<SysShipResVO> getLocation(@RequestParam(defaultValue = "true", name = "direction") Boolean direction, @RequestParam(defaultValue = "",name = "lat") String lat, @RequestParam(defaultValue = "",name = "lon") String lon )
    {
        return R.data(iAppService.findAllLocationByNext(direction,lat,lon));
    }

    @GetMapping("/location/wharf")
    @ApiOperation(value = "获取全部船只位于站点相对位置", notes = "获取全部船只位于站点相对位置", produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "direction", value = "正向（TRUE）/反向（FALSE）、默认正向", required = true, dataType = "Boolean"),
            @ApiImplicitParam(name = "lat", value = "纬度", required = true, dataType = "String"),
            @ApiImplicitParam(name = "lon", value = "经度", required = true, dataType = "String")})
    public R<SysShipExtendListResVO> getLocationWharf(@RequestParam(defaultValue = "true", name = "direction") Boolean direction, @RequestParam(defaultValue = "",name = "lat") String lat, @RequestParam(defaultValue = "",name = "lon") String lon )
    {
        return R.data(iAppService.findAllLocation(direction,lat,lon));
    }


    @GetMapping("/location/all")
    @ApiOperation(value = "获取全部船只位置", notes = "获取全部船只位置", produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "direction", value = "正向（TRUE）/反向（FALSE）、默认正向", required = true, dataType = "Boolean")})
    public R<List<SysShipExtendVO>> getLocationAll(@RequestParam(defaultValue = "true", name = "direction") Boolean direction)
    {
        return R.data(iAppService.findAllLocation(direction));
    }

    @GetMapping("/wharf/my")
    @ApiOperation(value = "获取离我最近的站点及距离", notes = "获取离我最近的站点及距离", produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "lat", value = "纬度", required = true, dataType = "String"),
            @ApiImplicitParam(name = "lon", value = "经度", required = true, dataType = "String")})
    public R<SysWharfVO> getWharfMy(@RequestParam(defaultValue = "",name = "lat") String lat, @RequestParam(defaultValue = "",name = "lon") String lon )
    {
        return R.data(iAppService.findNearestStation(lat,lon));
    }

    @GetMapping("/route")
    @ApiOperation(value = "获取航线信息", notes = "获取航线信息", produces = "application/json")
    public R<List<SysRouteVo>> getRoute()
    {
        return R.data(sysWharfService.getRouteList());
    }

    @GetMapping("/openid")
    @ApiOperation(value = "根据code获取openid", notes = "根据code获取openid", produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "临时code", required = true, dataType = "String")})
    public R getOpenid(@RequestParam(name = "code") String code){
        String openid = null;
        String msg = null;
		try {
            WxMaJscode2SessionResult session = wxMaService.getUserService().getSessionInfo(code);
            openid = session.getOpenid();
        } catch (WxErrorException e) {
            e.printStackTrace();
            msg = e.getMessage();
            // 处理异常，例如返回错误信息给前端
        }
        return R.data(openid, msg);
	}

    @GetMapping("/reservation/config")
    @ApiOperation(value = "获取预约配置", notes = "获取预约配置", produces = "application/json")
    public R<JSONArray> getReservationConfig()
    {
        return R.data(iAppService.getEnableReservationConfig());
    }



    @PostMapping("/reservation/data")
    @ApiOperation(value = "预约提交", notes = "预约提交", produces = "application/json")
    public R saveReservation(@RequestBody @Validated SysReservationData sysReservationData)
    {
        return iAppService.saveReservationData(sysReservationData);
    }



    @PostMapping("/reservation/cancel")
    @ApiOperation(value = "取消预约", notes = "取消预约", produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "预约记录id", required = true, dataType = "String")})
    public R cancelReservation(@RequestParam(name = "id") String id)
    {
        return R.status(iAppService.cancelReservation(id));
    }

    @GetMapping("/reservation/history")
    @ApiOperation(value = "获取历史预约", notes = "获取历史预约", produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "openid", value = "小程序id", required = true, dataType = "String"),
            @ApiImplicitParam(name = "type", value = "类型 0为 未使用 1为 已过期", required = true, dataType = "String"),
    })
    public R<List<SysReservationDataHistoryVO>> getHistoryReservationList(@RequestParam(name = "openid") String openid,
                                                                          @RequestParam(defaultValue = "0" ,name = "type") String type)
    {
        List<SysReservationDataHistoryVO> list = iAppService.getHistoryReservationList(openid, type);
        return R.data(list);
    }

    @GetMapping("/notice/type")
    @ApiOperation(value = "获取信息类型", notes = "获取信息类型", produces = "application/json")
    public R<List<SysDictData>> getNoticeType()
    {
        SysDictData sysDictData = new SysDictData();
        sysDictData.setDictType("sys_notice_type");
        List<SysDictData> list = sysDictDataService.findList(sysDictData);
        return R.data(list);
    }

    @GetMapping("/notice/data")
    @ApiOperation(value = "按信息类型获取信息数据", notes = "按信息类型获取信息数据", produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dictValue", value = "信息类型", required = true, dataType = "String"),
            @ApiImplicitParam(name = "pageNum", value = "页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pageSize", value = "页大小", required = true, dataType = "int")})
    public R<PageInfo> getNoticeData(@RequestParam(defaultValue = "1",name = "dictValue") String dictValue,
                                     @RequestParam(defaultValue = "1" ,name = "pageNum") Integer pageNum,
                                     @RequestParam(defaultValue = "10" ,name = "pageSize") Integer pageSize)
    {
        SysNotice sysNotice = new SysNotice();
        sysNotice.setNoticeType(dictValue);
        sysNotice.setStatus(BaseEntity.STATUS_NORMAL);
        PageDomain pageDomain = new PageDomain();
        pageDomain.setPageNum(pageNum);
        pageDomain.setPageSize(pageSize);
        sysNotice.setPage(pageDomain);
        return R.data(noticeService.findPage(sysNotice));
    }

    @PostMapping("/message")
    public R getMessage(@RequestBody Message message, HttpServletRequest request, HttpServletResponse response)
    {
        // 验证签名时使用
        // String timestamp = request.getHeader("timestamp");
        // String signature = request.getHeader("signature");
        // String type = request.getHeader("type");
        // String device_sn = request.getHeader("device_sn");
        // String message_id = request.getHeader("message_id");
        // logger.info("webhook消息推送，消息-------{}", message);
        Map<String, Object> extend = message.getExtend();
        logger.info("webhook消息推送，消息----extend---{}", extend);
        if (null != extend) {
            logger.info("webhook消息推送，消息----extend-count--{}", extend.containsKey("count"));
            if (extend.containsKey("count")) {
                ArrayList<Object> alarm_count = (ArrayList<Object>) extend.get("count");
                logger.info("webhook消息推送，消息----extend-alarm_count--{}", alarm_count);
                if (alarm_count.size() > 0){
                    logger.info("webhook消息推送，告警人数----人数---{}", alarm_count.get(0));
                    logger.info("webhook消息推送，请求头----时间戳---{}", message.getTimestamp());
                    logger.info("webhook消息推送，请求头----设备号---{}", message.getDevice_sn());
                    DateTime date = DateUtil.date(Long.valueOf(message.getTimestamp()) * 1000);
                    logger.info("webhook消息推送，请求头----时间戳转换---{}", date);
                    int row = iAppService.insertCameraAlarm(new SysCameraAlarm(IdUtils.fastSimpleUUID(), date, message.getDevice_sn(), String.valueOf(alarm_count.get(0))));
                    return R.status(row);
                }
            }
        }
        return R.status(false);
    }


    @GetMapping("/test")
    @ApiOperation(value = "获取信息类型", notes = "获取信息类型", produces = "application/json")
    public R test()
    {
        sysShipService.getShipSurplusSeat("261836442bbb48659d40f716feaa470d", 300);
        return R.status(true);
    }

}
