package com.aidex.web.controller.system;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.analysis.WxMaRetainInfo;
import cn.binarywang.wx.miniapp.bean.analysis.WxMaSummaryTrend;
import cn.hutool.core.date.DateUtil;
import com.aidex.common.config.AiDexConfig;
import com.aidex.common.core.domain.R;
import com.aidex.common.utils.StringUtils;
import com.aidex.system.service.SysAlarmService;
import com.aidex.system.service.SysShipService;
import com.alibaba.fastjson2.JSONObject;
import me.chanjar.weixin.common.error.WxErrorException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.List;

/**
 * 首页
 *
 * @author ruoyi
 */
@RestController
public class SysIndexController
{
    /** 系统基础配置 */
    @Autowired
    private AiDexConfig aidexConfig;

    @Autowired
    private SysShipService sysShipService;

    @Autowired
    private SysAlarmService alarmService;

    @Autowired
    private WxMaService wxMaService;

    /**
     * 访问首页，提示语
     */
    @RequestMapping("/")
    public String index()
    {
        return StringUtils.format("欢迎使用{}后台管理框架，当前版本：v{}，请通过前端地址访问。", aidexConfig.getName(), aidexConfig.getVersion());
    }


    /**
     * @Description: 首页统计数据<br/>
     * @date: 2024/4/7 10:29<br/>
     * @param: <br />
     * @author: Chensir <br/>
     * @since JDK 1.8
     * @return
     */
    @GetMapping("/index/statistics")
    public R statistics()
    {
        // 在线船只
        int onlineShip = sysShipService.countSysShipByStatus();
        // 今日报警数量
        int todayAlarmNum = alarmService.countSysAlarmByDate();
        // 新增用户
        int uv_new_num = 0;
        // 活跃用户
        int uv_active_num = 0;
        // 总用户
        Long uv_all_num = 0L;
        // 用户活跃量
		try {
            // 获取日留存数据
			WxMaRetainInfo dailyRetainInfo = wxMaService.getAnalysisService().getDailyRetainInfo(DateUtil.yesterday(),DateUtil.yesterday());
            // 获取用户访问小程序数据概况
            List<WxMaSummaryTrend> dailySummaryTrend = wxMaService.getAnalysisService().getDailySummaryTrend(DateUtil.yesterday(), DateUtil.yesterday());
            if (dailyRetainInfo != null){
                uv_new_num = dailyRetainInfo.getVisitUvNew().get(0);
                uv_active_num = dailyRetainInfo.getVisitUv().get(0);
            }
            if (dailySummaryTrend != null && dailySummaryTrend.size() > 0){
                uv_all_num = dailySummaryTrend.get(0).getVisitTotal();
            }
		} catch (WxErrorException e) {
			throw new RuntimeException(e);
		}
        JSONObject data = new JSONObject();
        data.put("onlineShip", onlineShip);
        data.put("todayAlarmNum", todayAlarmNum);
        data.put("uv_new_num", uv_new_num);
        data.put("uv_active_num", uv_active_num);
        data.put("uv_all_num", uv_all_num);

		return R.data(data);
    }

}
