package com.aidex.web.controller.system;

import java.util.List;
import java.util.HashMap;
import java.util.Map;
import com.aidex.common.annotation.Log;
import com.aidex.common.core.domain.R;
import com.aidex.system.req.SysReservationConfigReq;
import com.github.pagehelper.PageInfo;
import com.aidex.common.core.page.PageDomain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.aidex.common.enums.BusinessType;
import com.aidex.common.utils.poi.ExcelUtil;
import com.aidex.framework.cache.DictUtils;
import javax.validation.constraints.*;
import org.springframework.web.bind.annotation.*;
import com.aidex.common.core.controller.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import com.aidex.system.domain.SysReservationConfig;
import com.aidex.system.service.SysReservationConfigService;

/**
 * 预约配置Controller
 * @author Chensir
 * @email 914769835
 * @date 2024-03-28
 */
@RestController
@RequestMapping("/system/sysReservationConfig")
public class SysReservationConfigController extends BaseController {

    @Autowired
    private SysReservationConfigService sysReservationConfigService;

    /**
     * 查询预约配置列表
     */
    @PreAuthorize("@ss.hasPermi('system:sysReservationConfig:list')")
    @GetMapping("/list")
    public R<PageInfo> list(SysReservationConfig sysReservationConfig, HttpServletRequest request, HttpServletResponse response) {
        sysReservationConfig.setPage(new PageDomain(request, response));
        return R.data(sysReservationConfigService.findPage(sysReservationConfig));
    }

    /**
     * 获取预约配置详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:sysReservationConfig:query')")
    @GetMapping(value = "/{id}")
    public R<SysReservationConfig> detail(@PathVariable("id") String id) {
        return R.data(sysReservationConfigService.get(id));
    }

    @PreAuthorize("@ss.hasPermi('system:sysReservationConfig:add')")
    @Log(title = "预约配置", businessType = BusinessType.INSERT)
    @PostMapping("/batchAdd")
    public R addBySysReservationConfigReq(@RequestBody @Validated SysReservationConfigReq sysReservationConfigReq) {
        return R.status(sysReservationConfigService.addBySysReservationConfigReq(sysReservationConfigReq));
    }

    @PreAuthorize("@ss.hasPermi('system:sysReservationConfig:list')")
    @GetMapping("/statisc")
    public R<List<Map<String, String>>> statisc(@RequestParam(value = "date", required = true) String date) {
        List<Map<String, String>> statiscDataBydate = sysReservationConfigService.getStatiscDataBydate(date);
        return R.data(statiscDataBydate);
    }

    @GetMapping("/statisc/columns")
    public R<List<String>> statisColumns(@RequestParam(value = "date", required = true) String date) {
        return R.data(sysReservationConfigService.getStatiscColumns(date));
    }

    /**
     * 新增预约配置
     */
    @PreAuthorize("@ss.hasPermi('system:sysReservationConfig:add')")
    @Log(title = "预约配置", businessType = BusinessType.INSERT)
    @PostMapping
    public R add(@RequestBody @Validated  SysReservationConfig sysReservationConfig) {
        return R.status(sysReservationConfigService.save(sysReservationConfig));
    }

    /**
     * 修改预约配置
     */
    @PreAuthorize("@ss.hasPermi('system:sysReservationConfig:edit')")
    @Log(title = "预约配置", businessType = BusinessType.UPDATE)
    @PutMapping
    public R edit(@RequestBody @Validated SysReservationConfig sysReservationConfig) {
        return R.status(sysReservationConfigService.save(sysReservationConfig));
    }

    /**
     * 删除预约配置
     */
    @PreAuthorize("@ss.hasPermi('system:sysReservationConfig:remove')")
    @Log(title = "预约配置", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R remove(@PathVariable String[] ids) {
        return R.status(sysReservationConfigService.deleteSysReservationConfigByIds(ids));
    }

    /**
     * 按日期删除预约配置
     */
    @PreAuthorize("@ss.hasPermi('system:sysReservationConfig:remove')")
    @Log(title = "预约配置", businessType = BusinessType.DELETE)
    @DeleteMapping("/removeByDate")
    public R removeByDate(@RequestParam(value = "date", required = true) String date) {
        return sysReservationConfigService.removeByDate(date);
    }


    /**
     * 导出预约配置列表
     */
    @PreAuthorize("@ss.hasPermi('system:sysReservationConfig:export')")
    @Log(title = "预约配置", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public R export(SysReservationConfig sysReservationConfig) {
        List<SysReservationConfig> list = sysReservationConfigService.findList(sysReservationConfig);
        ExcelUtil<SysReservationConfig> util = new ExcelUtil<SysReservationConfig>(SysReservationConfig.class);
        return util.exportExcel(list, "预约配置数据");
    }

    /**
     * 根据字典类型查询字典数据信息等其他自定义信息
     */
    @GetMapping(value = "/getInitData/{dictTypes}")
    public R getInitData(@PathVariable String dictTypes) {
        Map<String, Object> dataMap = new HashMap<String, Object>();
        dataMap.putAll(DictUtils.getMultiDictList(dictTypes));
        return R.data(dataMap);
    }

    /**
     * 按日期修改预约配置
     */
    @PreAuthorize("@ss.hasPermi('system:sysReservationConfig:edit')")
    @Log(title = "预约配置", businessType = BusinessType.UPDATE)
    @PutMapping("/editByReservationDate")
    public R editByReservationDate(@RequestBody  SysReservationConfig sysReservationConfig) {
        return sysReservationConfigService.editByReservationDate(sysReservationConfig);
    }

}
