package com.aidex.common.app.server.impl;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.aidex.common.app.domain.vo.SysWharfVO;
import com.aidex.common.app.server.IAppService;
import com.aidex.common.constant.CacheConstants;
import com.aidex.common.constant.Constants;
import com.aidex.common.core.domain.BaseEntity;
import com.aidex.common.core.domain.R;
import com.aidex.common.core.redis.RedisCache;
import com.aidex.common.exception.SysException;
import com.aidex.common.gps.server.IGpsService;
import com.aidex.common.plush.domain.vo.SysShipExtendListResVO;
import com.aidex.common.plush.domain.vo.SysShipExtendListVO;
import com.aidex.common.plush.domain.vo.SysShipExtendVO;
import com.aidex.common.plush.domain.vo.SysShipResVO;
import com.aidex.common.plush.service.SysShipExtendService;
import com.aidex.common.utils.StringUtils;
import com.aidex.common.utils.dist.CalcDist;
import com.aidex.common.utils.uuid.IdUtils;
import com.aidex.framework.cache.ConfigUtils;
import com.aidex.system.domain.*;
import com.aidex.system.domain.vo.SysReservationDataHistoryVO;
import com.aidex.system.domain.vo.SysReservationDataNumVO;
import com.aidex.system.mapper.SysCameraAlarmMapper;
import com.aidex.system.mapper.SysReservationDataMapper;
import com.aidex.system.service.SysNoticeService;
import com.aidex.system.service.SysReservationConfigService;
import com.aidex.system.service.SysShipService;
import com.aidex.system.service.SysWharfService;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class AppService implements IAppService {

    @Autowired
    private SysNoticeService sysNoticeService;

    @Autowired
    private SysWharfService sysWharfService;

    @Autowired
    private SysShipExtendService sysShipExtendService;

    @Autowired
    private SysReservationConfigService sysReservationConfigService;

    @Autowired
    private SysReservationDataMapper sysReservationDataMapper;

    @Autowired
    private SysCameraAlarmMapper sysCameraAlarmMapper;

    @Autowired
    private RedisCache redisCache;


    public List<SysNotice> getCarouselList(){

        List<SysNotice> carouselList = sysNoticeService.findList(new SysNotice(){{setCarousel("1"); setStatus(BaseEntity.STATUS_NORMAL);}});
        return carouselList;
    }

    public List<SysNotice> getNoticeList(){
        List<SysNotice> noticeList = sysNoticeService.findList(new SysNotice(){{setCarousel("0");setNoticeType("1");setStatus(BaseEntity.STATUS_NORMAL);}});
        return noticeList;
    }

    public SysNotice findCarouselAndNoticeDetails(String id){
        return sysNoticeService.get(id);
    }

    public List<SysWharf> findSysWharfList(){
        return sysWharfService.findList(new SysWharf());
    }

    /*
    * direction 方向 true正向 false反向
    * */
    public List<SysShipExtendVO> findAllLocation(Boolean direction){
        List<SysShipExtendVO> sysShipExtendVOS = sysShipExtendService.findAllonline(direction?"1":"0");
        if(sysShipExtendVOS.size() <= 0)
            throw new SysException(8821,"暂无运行船只");
        return sysShipExtendVOS;
    }


    public SysShipResVO findAllLocationByNext(Boolean direction,String lat, String lon){
        if(StringUtils.isEmpty(lat) || StringUtils.isEmpty(lon))
            throw new SysException(4441,"请告诉我你在哪");

        String travel = direction?"1":"0";
        // 离我最近的站点
        SysWharf wharf = this.findNearestStation(lat,lon);

        if(!wharf.getStatus().equals("0"))
            throw new SysException(4442, wharf.getWharfNanme()+" 已经停用！请更换其他站点");

        List<SysShipExtendVO> sysShipExtendVOS = sysShipExtendService.findAllonline(travel).stream().map(lot -> {
                    lot.setDistance(CalcDist.calculateDistance(Double.valueOf(wharf.getLat()), Double.valueOf(wharf.getLng()),
                            Double.valueOf(lot.getLat()), Double.valueOf(lot.getLon())));
                    return lot;
                }).sorted(Comparator.comparing(SysShipExtendVO::getDistance))
                .collect(Collectors.toList());

        if(sysShipExtendVOS.size() <= 0)
            throw new SysException(8821,"暂无运行船只");


        // 默认到站停留时间
        Integer inStopTime = Integer.valueOf(ConfigUtils.getConfigByKey("pull.in.time").getConfigValue());
        // 停靠速度
        Integer stopSpeed = Integer.valueOf(ConfigUtils.getConfigByKey("stop.speed").getConfigValue());
        // 停靠误差范围
        Integer stopRadius = Integer.valueOf(ConfigUtils.getConfigByKey("stop.ship.radius").getConfigValue());

        // 查找未经过该站点的船只
        SysShipExtendVO noSiteShip = null;
        for (SysShipExtendVO sysShipExtendVO : sysShipExtendVOS) {
            if(direction && CalcDist.gps2d(Double.valueOf(wharf.getLat()),Double.valueOf(wharf.getLng()),Double.valueOf(sysShipExtendVO.getLat()),Double.valueOf(sysShipExtendVO.getLon())) <= Double.valueOf(stopRadius)){
                noSiteShip = sysShipExtendVO;
                break;
            }
            if(!direction && CalcDist.gps2d(Double.valueOf(sysShipExtendVO.getLat()),Double.valueOf(sysShipExtendVO.getLon()),Double.valueOf(wharf.getLat()),Double.valueOf(wharf.getLng())) <= Double.valueOf(stopRadius)){
                noSiteShip = sysShipExtendVO;
                break;
            }
        }



        if(noSiteShip == null)
            throw new SysException(8221,"当前运行船只已全部经过该站点！暂无船只！请等待发船");
        SysShipResVO sysShipResVO = new SysShipResVO();
        BeanUtils.copyProperties(noSiteShip,sysShipResVO);
        sysShipResVO.setIsStopMyLocationSite(Boolean.FALSE);
        sysShipResVO.setMy(wharf);

        Long defaultSpeed = 0L;
        if(sysShipResVO.getTravel() == 1L) {
            defaultSpeed = Long.valueOf(ConfigUtils.getConfigByKey("forward.default.speed").getConfigValue());
        }else {
            defaultSpeed = Long.valueOf(ConfigUtils.getConfigByKey("reverse.default.speed").getConfigValue());
        }
        // 停靠时间
        Long awaitTime = 0L;
        Double stopDist = 0.0;
        Long distSiteNum = 0L;

        SysWharf wharfShipStop = null;
        SysWharf wharfShipNext = null;

        // 船只已经停止
        if(sysShipResVO.getIsStop() == 1L){
            wharfShipStop = sysWharfService.get(sysShipResVO.getWharfStop());
            sysShipResVO.setStop(wharfShipStop);
            // 停靠同一个站点
            if(wharfShipStop.getWharfNanme().equals(wharf.getWharfNanme())){
                sysShipResVO.setIsStopMyLocationSite(Boolean.TRUE);
            }else {
                //如果处于停止状态则计算停留剩余时间
                Double loadingTime = 0.0;
                if(sysShipResVO.getUpdateTime() != null && sysShipResVO.getUpdateTime() < System.currentTimeMillis())
                    loadingTime = inStopTime.doubleValue() - CalcDist.calcStopTime(sysShipResVO.getUpdateTime().toString());
                // 计算站点间距离
                stopDist = CalcDist.calculateDistance(Double.valueOf(wharf.getLat()), Double.valueOf(wharf.getLng()),Double.valueOf(wharfShipStop.getLat()), Double.valueOf(wharfShipStop.getLng()));
                Double time = Math.ceil(((stopDist / 1000) / Double.valueOf(defaultSpeed)) * 60);
                awaitTime = time.longValue() + loadingTime.longValue();
                // 船只正向反向距离站点数量
                Long myNum = wharf.getWharfOrder() / 10;
                Long shipNum = wharfShipStop.getWharfOrder() / 10;
                if(sysShipResVO.getTravel() == 1L)
                    distSiteNum = shipNum - myNum;
                else
                    distSiteNum = myNum - shipNum;
            }
        }else {
            wharfShipNext = sysWharfService.get(sysShipResVO.getNextWharf());
            stopDist = CalcDist.calculateDistance(Double.valueOf(wharf.getLat()), Double.valueOf(wharf.getLng()),Double.valueOf(sysShipResVO.getLat()), Double.valueOf(sysShipResVO.getLon()));
            Double time = 0.0;
            if(Double.valueOf(sysShipResVO.getSpeed()) > Double.valueOf(stopSpeed))
                time = Math.ceil(((stopDist / 1000) / Double.valueOf(sysShipResVO.getSpeed())) * 60);
            else
                time = Math.ceil(((stopDist / 1000) / Double.valueOf(defaultSpeed)) * 60);
            awaitTime = time.longValue();
            sysShipResVO.setNext(wharfShipNext);
            // 船只正向反向距离站点数量
            Long myNum = wharf.getWharfOrder() / 10;
            Long shipNum = wharfShipNext.getWharfOrder() / 10;
            if(sysShipResVO.getTravel() == 1L)
                distSiteNum = shipNum - myNum;
            else
                distSiteNum = myNum - shipNum;
        }
        String numKey = redisCache.get(CacheConstants.SYS_SURPLUS_KEY+sysShipResVO.getShipId()).toString();
        Long resetNum = 0L;
        if(numKey == null)
            resetNum = -1L;
        else {
            Object num = redisCache.get(numKey);
            resetNum = num == null ? -1L: Long.valueOf(num.toString());
        }

        sysShipResVO.setReseat(resetNum);
        sysShipResVO.setAwaitTime(awaitTime);
        sysShipResVO.setDistance(stopDist);
        sysShipResVO.setDistSiteNum(distSiteNum);

        return sysShipResVO;
    }


    public SysShipExtendListResVO findAllLocation(Boolean direction, String lat, String lon){
        if(StringUtils.isEmpty(lat) || StringUtils.isEmpty(lon))
            throw new SysException(4441,"请告诉我你在哪");

        String travel = direction?"1":"0";
        // 离我最近的站点
        SysWharf wharf = this.findNearestStation(lat,lon);

        if(!wharf.getStatus().equals("0"))
            throw new SysException(4442, wharf.getWharfNanme()+" 已经停用！请更换其他站点");

        List<SysShipExtendVO> sysShipExtendVOS = sysShipExtendService.findAllonline(travel);

        if(sysShipExtendVOS.size() <= 0)
            throw new SysException(8821,"暂无运行船只");

        List<SysShipExtendListVO> sysShipExtendListVOS = new ArrayList<>();
        for (SysShipExtendVO sysShipExtendVO : sysShipExtendVOS) {
            SysShipExtendListVO sysShipExtendListVO = new SysShipExtendListVO();
            BeanUtils.copyProperties(sysShipExtendVO,sysShipExtendListVO);
            sysShipExtendListVO.setMy(wharf);
            // 已经停靠
            if(sysShipExtendVO.getIsStop().equals(1L)){
                sysShipExtendListVO.setStop(sysWharfService.get(sysShipExtendVO.getWharfStop()));
            }else {
                sysShipExtendListVO.setNext(sysWharfService.get(sysShipExtendVO.getNextWharf()));
            }
            sysShipExtendListVOS.add(sysShipExtendListVO);
        }

        SysShipExtendListResVO resVO = new SysShipExtendListResVO();
        resVO.setShipList(sysShipExtendListVOS);

        try {
            SysShipResVO sysShipResVO = this.findAllLocationByNext(direction,lat,lon);
            resVO.setShipLocation(sysShipResVO);
        }catch (Exception e){
            resVO.setShipLocation(null);
        }
        return resVO;
    }

    public SysWharfVO findNearestStation(String lat, String lon){
        if(StringUtils.isEmpty(lat) || StringUtils.isEmpty(lon))
            throw new SysException(4441,"请告诉我你在哪");
        List<SysWharf> listWharf = sysWharfService.findListByCache(new SysWharf());
        List<SysWharfVO> vos = new ArrayList<>();
        for (SysWharf sysWharf : listWharf) {
            SysWharfVO wharfVO = new SysWharfVO();
            BeanUtils.copyProperties(sysWharf,wharfVO);
            wharfVO.setDistance(CalcDist.calculateDistance(Double.valueOf(lat),Double.valueOf(lon),
                    Double.valueOf(sysWharf.getLat()),Double.valueOf(sysWharf.getLng())));
            vos.add(wharfVO);
        }
        return  vos.stream().sorted(Comparator.comparing(SysWharfVO::getDistance)).findFirst().orElse(null);
    }

    @Override
    public JSONArray getEnableReservationConfig() {
        JSONArray jsonArray = new JSONArray();
        SysConfig beforeDaysConfig =  ConfigUtils.getConfigByKey("sys.reservation.before");
        SysConfig betweenDayConfig = ConfigUtils.getConfigByKey("sys.reservation.days");
        if(beforeDaysConfig != null && betweenDayConfig != null){
            Integer beforeDays = Integer.valueOf(beforeDaysConfig.getConfigValue());
            Integer betweenDay = Integer.valueOf(betweenDayConfig.getConfigValue());
            if (beforeDays > 0){
                DateTime startDate = DateUtil.offset(new Date(), DateField.DAY_OF_YEAR, beforeDays);
                DateTime endDate =  DateUtil.offset(startDate, DateField.DAY_OF_YEAR, betweenDay);
                SysReservationConfig sysReservationConfig = new SysReservationConfig();
                sysReservationConfig.setBeginReservationDate(DateUtil.formatDate(startDate));
                sysReservationConfig.setEndReservationDate(DateUtil.formatDate(endDate));
                sysReservationConfig.setStatus("1");
                List<SysReservationConfig> list = sysReservationConfigService.findList(sysReservationConfig);
                List<String> sysReservationConfigIds = list.stream().map(SysReservationConfig::getId).collect(Collectors.toList());
                List<SysReservationDataNumVO> numByReservationConfigIds = sysReservationDataMapper.getNumByReservationConfigIds(sysReservationConfigIds.toArray(new String[]{}));
                SysReservationDataNumVO defaultMap = new SysReservationDataNumVO();
                defaultMap.setNum(0L);
                list.stream().forEach(item -> item.setEnableNum(item.getEnableNum() - numByReservationConfigIds.stream().filter(item1 -> item1.getReservationConfigId().equals(item.getId())).findFirst().orElse(defaultMap).getNum()));
                Map<Date, List<SysReservationConfig>> collect = list.stream().collect(Collectors.groupingBy(SysReservationConfig::getReservationDate, Collectors.mapping(value -> value, Collectors.toList())));
                collect.forEach((k,v)->{
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("date",DateUtil.formatDate(k));
                    // 计算可预约人数
                    jsonObject.put("list",v.stream().sorted(Comparator.comparing(SysReservationConfig::getReservationStartTime)).collect(Collectors.toList()));
                    jsonArray.add(jsonObject);
                });
            } else {
                DateTime startDate = DateUtil.date();
                DateTime endDate =  DateUtil.offset(startDate, DateField.DAY_OF_YEAR, betweenDay);
                SysReservationConfig sysReservationConfig = new SysReservationConfig();
                sysReservationConfig.setBeginReservationDate(DateUtil.formatDate(startDate));
                sysReservationConfig.setEndReservationDate(DateUtil.formatDate(endDate));
                sysReservationConfig.setBeginReservationStartTime(DateUtil.formatDateTime(DateUtil.date()));
                sysReservationConfig.setStatus("1");
                List<SysReservationConfig> list = sysReservationConfigService.findList(sysReservationConfig);
                List<String> sysReservationConfigIds = list.stream().map(SysReservationConfig::getId).collect(Collectors.toList());
                List<SysReservationDataNumVO> numByReservationConfigIds = sysReservationDataMapper.getNumByReservationConfigIds(sysReservationConfigIds.toArray(new String[]{}));
                SysReservationDataNumVO defaultMap = new SysReservationDataNumVO();
                defaultMap.setNum(0L);
                list.stream().forEach(item -> item.setEnableNum(item.getEnableNum() - numByReservationConfigIds.stream().filter(item1 -> item1.getReservationConfigId().equals(item.getId())).findFirst().orElse(defaultMap).getNum()));
                Map<Date, List<SysReservationConfig>> collect = list.stream().collect(Collectors.groupingBy(SysReservationConfig::getReservationDate, Collectors.mapping(value -> value, Collectors.toList())));
                collect.forEach((k,v)->{
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("date",DateUtil.formatDate(k));
                    // 计算可预约人数
                    jsonObject.put("list",v.stream().sorted(Comparator.comparing(SysReservationConfig::getReservationStartTime)).collect(Collectors.toList()));
                    jsonArray.add(jsonObject);
                });
            }
        }
        jsonArray.sort(Comparator.comparing(obj -> ((JSONObject) obj).getDate("date")));
        return jsonArray;
    }

    @Override
    public R saveReservationData(SysReservationData sysReservationData) {
        if (sysReservationDataMapper.getNumByOpenid(sysReservationData) > 0){
            return R.fail("当前预约日期已存在预约记录！");
        }
        int numByReservationConfigId = sysReservationDataMapper.getNumByReservationConfigId(sysReservationData);
        SysReservationConfig sysReservationConfig = sysReservationConfigService.get(sysReservationData.getReservationConfigId());
        if (sysReservationConfig!=null && numByReservationConfigId >= sysReservationConfig.getEnableNum()){
            return R.fail("已超出可预约人数！");
        }
        sysReservationData.setReservationTime(DateUtil.date());
        sysReservationData.setId(IdUtils.simpleUUID());
        // 判断预约记录
        return R.status(sysReservationDataMapper.addReservationData(sysReservationData));
    }

    @Override
    public int cancelReservation(String id) {
        return sysReservationDataMapper.deleteById(id);
    }

    @Override
    public List<SysReservationDataHistoryVO> getHistoryReservationList(String openid, String type) {
        List<SysReservationDataHistoryVO> historyData = sysReservationDataMapper.getHistoryData(openid, type, DateUtil.formatDate(DateUtil.date()));
        SysConfig configByKey = ConfigUtils.getConfigByKey("sys.before.open.minte");
        if (configByKey != null){
            String value = configByKey.getConfigValue();
            if (!Constants.SUCCESS.equals(value)){
                historyData.stream().forEach(item -> item.setBeforeOpenTime(DateUtil.offsetDay(item.getReservationTime(), -1)));
            }
        }
        return historyData;
    }

    @Override
    public int insertCameraAlarm(SysCameraAlarm sysCameraAlarm) {
        return sysCameraAlarmMapper.insertCameraAlarm(sysCameraAlarm);
    }

}
