package com.aidex.common.gps.server.impl;

import com.aidex.common.app.domain.vo.SysWharfVO;
import com.aidex.common.constant.CacheConstants;
import com.aidex.common.core.redis.RedisCache;
import com.aidex.common.gps.domain.PushAlarmEntity;
import com.aidex.common.gps.domain.PushLocationEntity;
import com.aidex.common.gps.server.IGpsPlushService;
import com.aidex.common.plush.domain.GpsAlarmPlush;
import com.aidex.common.plush.domain.GpsLocationPlush;
import com.aidex.common.plush.domain.SysShipExtend;
import com.aidex.common.plush.service.GpsAlarmPlushService;
import com.aidex.common.plush.service.GpsLocationPlushService;
import com.aidex.common.plush.service.SysShipExtendService;
import com.aidex.common.utils.StringUtils;
import com.aidex.common.utils.dist.CalcDist;
import com.aidex.common.utils.file.ImageUtils;
import com.aidex.framework.cache.CacheUtil;
import com.aidex.framework.cache.ConfigUtils;
import com.aidex.system.domain.SysWharf;
import com.aidex.system.service.SysShipService;
import com.aidex.system.service.SysWharfService;
import com.alibaba.fastjson2.JSONArray;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;


@Service
public class GpsPlushService implements IGpsPlushService {

    private static final Logger log = LoggerFactory.getLogger(ImageUtils.class);

    @Autowired
    private SysShipExtendService sysShipExtendService;
    @Autowired
    private SysWharfService sysWharfService;
    @Autowired
    private GpsLocationPlushService gpsLocationPlushService;
    @Autowired
    private GpsAlarmPlushService gpsAlarmPlushService;
    @Autowired
    private SysShipService sysShipService;
    @Autowired
    private RedisCache redisCache;

    public void plush(String method, String serialNumber, String data){
        try {
            // 只接收报警和位置  其他数据丢弃
            if(method.equals(METHOD.ALARM)){
                List<PushAlarmEntity> pushAlarmEntities = JSONArray.parseArray(data).toList(PushAlarmEntity.class);
                alarmSource(pushAlarmEntities);
            }
            if(method.equals(METHOD.LOCATION)){
                List<PushLocationEntity> pushAlarmEntities = JSONArray.parseArray(data).toList(PushLocationEntity.class);
                locationSource(pushAlarmEntities);
            }
        }catch (Exception e){
            log.error("PLUSH_GPS: 推送数据接收错误！错误原因："+e.getMessage());
            log.error("PLUSH_GPS: 推送数据方法："+method+" 推送数据： "+ data);
        }

    }

    private void alarmSource(List<PushAlarmEntity> alarmEntities){
        List<SysWharf> sysWharfs = sysWharfService.findListByCache(new SysWharf());
        List<SysShipExtend> sysShipExtends = sysShipExtendService.findListByCache(new SysShipExtend());
        String[] forward = ConfigUtils.getConfigByKey("forward.direction").getConfigValue().split("-");
        // 获取反向条件
        String[] reverse = ConfigUtils.getConfigByKey("reverse.direction").getConfigValue().split("-");
        if(sysShipExtends.size() > 0){
            for (PushAlarmEntity alarmEntity : alarmEntities) {
                SysShipExtend updateShipExtend = sysShipExtends.stream().filter(extend -> alarmEntity.getMacid().contains(extend.getMacId())).findFirst().orElse(null);
                // 设备是当前设备  并且方位正常则更新数据
                if(alarmEntity.getClassify().equals(CLASSIFY.OUT) || alarmEntity.getClassify().equals(CLASSIFY.IN)) {
                    if (updateShipExtend != null &&
                            ((Double.valueOf(alarmEntity.getDir()) >= Double.valueOf(forward[0]) && Double.valueOf(alarmEntity.getDir()) <= Double.valueOf(forward[1]))) ||
                            (Double.valueOf(alarmEntity.getDir()) >= Double.valueOf(reverse[0]) && Double.valueOf(alarmEntity.getDir()) <= Double.valueOf(reverse[1]))) {
                        updateShipExtend.setSpeed(alarmEntity.getSpeed());
                        updateShipExtend.setDir(alarmEntity.getDir());
                        updateShipExtend.setLon(alarmEntity.getMapLon());
                        updateShipExtend.setLat(alarmEntity.getMapLat());
                        updateShipExtend.setWharfStop(null);
                        updateShipExtend.setNextWharf(null);
                        if (Double.valueOf(alarmEntity.getDir()) >= Double.valueOf(forward[0]) && Double.valueOf(alarmEntity.getDir()) <= Double.valueOf(forward[1]))
                            updateShipExtend.setTravel(1L);
                        if (Double.valueOf(alarmEntity.getDir()) >= Double.valueOf(reverse[0]) && Double.valueOf(alarmEntity.getDir()) <= Double.valueOf(reverse[1]))
                            updateShipExtend.setTravel(0L);
                        if (alarmEntity.getClassify().equals(CLASSIFY.IN)) {
                            SysWharfVO stopWharf = getLoccationNearly(sysWharfs,alarmEntity.getMapLat(),alarmEntity.getMapLon());
                            if (stopWharf != null) {
                                updateShipExtend.setIsStop(1L);
                                updateShipExtend.setWharfStop(stopWharf.getId());
                            }
                        }
                        if (alarmEntity.getClassify().equals(CLASSIFY.OUT)) {
                            String surpKey = sysShipService.getShipSurplusSeat(updateShipExtend.getShipId(),15);
                            redisCache.set(CacheConstants.SYS_SURPLUS_KEY+updateShipExtend.getShipId(),surpKey);
                            SysWharfVO stopWharf = getLoccationNearly(sysWharfs,alarmEntity.getMapLat(),alarmEntity.getMapLon());
                            if (stopWharf != null) {
                                updateShipExtend.setIsStop(0L);
                                // 正向行驶
                                if (updateShipExtend.getTravel() == 1L) {
                                    Long nextSite = (stopWharf.getWharfOrder() / 10) - 1;
                                    SysWharf nextWharf = sysWharfs.stream().filter(wharf -> wharf.getWharfOrder() == (nextSite * 10)).findFirst().orElse(null);
                                    updateShipExtend.setNextWharf(nextWharf.getId());
                                }
                                // 反向行驶
                                if (updateShipExtend.getTravel() == 0L) {
                                    Long nextSite = (stopWharf.getWharfOrder() / 10) + 1;
                                    SysWharf nextWharf = sysWharfs.stream().filter(wharf -> wharf.getWharfOrder() == (nextSite * 10)).findFirst().orElse(null);
                                    updateShipExtend.setNextWharf(nextWharf.getId());
                                }
                            }
                        }
                        updateShipExtend.setUpdateTime(System.currentTimeMillis());
                        sysShipExtendService.update(updateShipExtend);
                    }
                }

                GpsAlarmPlush alarmPlush = new GpsAlarmPlush();
                alarmPlush.setId(alarmEntity.getId());
                alarmPlush.setUserName(alarmEntity.getUserName());
                alarmPlush.setMacid(alarmEntity.getMacid());
                alarmPlush.setFullName(alarmEntity.getFullName());
                alarmPlush.setPTime(Long.valueOf(alarmEntity.getPTime()));
                alarmPlush.setAddTime(Long.valueOf(alarmEntity.getAddTime()));
                alarmPlush.setLat(alarmEntity.getLat());
                alarmPlush.setLon(alarmEntity.getLon());
                alarmPlush.setMapLat(alarmEntity.getMapLat());
                alarmPlush.setMapLon(alarmEntity.getMapLon());
                alarmPlush.setSpeed(alarmEntity.getSpeed());
                alarmPlush.setDir(alarmEntity.getDir());
                alarmPlush.setLat(alarmEntity.getLat());
                alarmPlush.setLon(alarmEntity.getLon());
                alarmPlush.setMapLat(alarmEntity.getMapLat());
                alarmPlush.setMapLon(alarmEntity.getMapLon());
                alarmPlush.setMapLon(alarmEntity.getMapLon());
                alarmPlush.setClassify(alarmEntity.getClassify());
                alarmPlush.setDescribe(alarmEntity.getDescribe());
                alarmPlush.setNotea(alarmEntity.getNotea());
                gpsAlarmPlushService.save(alarmPlush);
            }
        }

    }

    private void locationSource(List<PushLocationEntity> locationEntities){
        List<SysShipExtend> sysShipExtends = sysShipExtendService.findListByCache(new SysShipExtend());
        String[] forward = ConfigUtils.getConfigByKey("forward.direction").getConfigValue().split("-");
        // 获取反向条件
        String[] reverse = ConfigUtils.getConfigByKey("reverse.direction").getConfigValue().split("-");
        if(sysShipExtends.size() > 0) {
            for (PushLocationEntity location : locationEntities) {
                SysShipExtend updateShipExtend = sysShipExtends.stream().filter(extend -> location.getMacid().contains(extend.getMacId())).findFirst().orElse(null);
                // 设备是当前设备  并且方位正常则更新数据
                if (updateShipExtend != null &&
                        ((Double.valueOf(location.getDir()) >= Double.valueOf(forward[0]) && Double.valueOf(location.getDir()) <= Double.valueOf(forward[1]))) ||
                        (Double.valueOf(location.getDir()) >= Double.valueOf(reverse[0]) && Double.valueOf(location.getDir()) <= Double.valueOf(reverse[1]))) {
                    updateShipExtend.setSpeed(location.getSpeed());
                    updateShipExtend.setDir(location.getDir());
                    updateShipExtend.setLon(location.getMapLon());
                    updateShipExtend.setLat(location.getMapLat());
                    if(Double.valueOf(location.getSpeed()) > 0) {
                        if (Double.valueOf(location.getDir()) >= Double.valueOf(forward[0]) && Double.valueOf(location.getDir()) <= Double.valueOf(forward[1]))
                            updateShipExtend.setTravel(1L);
                        if (Double.valueOf(location.getDir()) >= Double.valueOf(reverse[0]) && Double.valueOf(location.getDir()) <= Double.valueOf(reverse[1]))
                            updateShipExtend.setTravel(0L);
                    }
                    String[] paramsValue = location.getValue() == null?new String[]{"0"}:location.getValue().split(",");
                    updateShipExtend.setMileage(paramsValue[0]);
                    sysShipExtendService.updateByMacId(updateShipExtend);
                }
                GpsLocationPlush locationPlush = new GpsLocationPlush();
                locationPlush.setMacid(location.getMacid());
                locationPlush.setGpsTime(Long.valueOf(location.getGpsTime()));
                locationPlush.setHeartTime(Long.valueOf(location.getHeartTime()));
                locationPlush.setUpdTime(location.getUpdTime());
                locationPlush.setSpeed(location.getSpeed());
                locationPlush.setDir(location.getDir());
                locationPlush.setLat(location.getLat());
                locationPlush.setLon(location.getLon());
                locationPlush.setMapLat(location.getMapLat());
                locationPlush.setMapLon(location.getMapLon());
                locationPlush.setStats(location.getStats());
                locationPlush.setValue(location.getValue());
                gpsLocationPlushService.save(locationPlush);
            }
        }
    }

    private SysWharfVO getLoccationNearly(List<SysWharf> sysWharves,String lat, String lon){
        return sysWharves.stream().map(wharf -> {
            SysWharfVO wharfVO = new SysWharfVO();
            BeanUtils.copyProperties(wharf,wharfVO);
            wharfVO.setDistance(CalcDist.calculateDistance(Double.valueOf(lat),Double.valueOf(lon),
                    Double.valueOf(wharf.getLat()),Double.valueOf(wharf.getLng())));
            return wharfVO;
        }).collect(Collectors.toList()).stream().sorted(Comparator.comparing(SysWharfVO::getDistance)).findFirst().orElse(null);
    }

    public interface METHOD {
        // 报警信息接收
        String ALARM = "alarm";
        // 位置信息接收
        String LOCATION = "status";
    }

    public interface CLASSIFY {
        // 入界
        String IN = "4";
        // 出界
        String OUT = "3";
    }
}
