package com.aidex.framework.cache;

import com.aidex.common.constant.CacheConstants;
import com.aidex.common.core.redis.RedisCache;
import com.aidex.common.plush.domain.SysShipExtend;
import com.aidex.common.plush.service.SysShipExtendService;
import com.aidex.common.utils.spring.SpringUtils;
import com.aidex.system.domain.SysWharf;
import com.aidex.system.service.SysWharfService;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.List;

/**
 * 配置工具类
 *
 * @author ruoyi
 */
@Component
public class ShipExtendUtils implements CacheUtil {

    @Override
    public String getCacheName() {
        return CACHE_NAME;
    }

    @Override
    public String getRemark() {
        return "船只扩展数据信息";
    }

    @Override
    public String getCacheId() {
        return "shipExtendUtils";
    }

    @Override
    public String getCacheValue(String key) {
        return REDIS_CACHE.getCacheMapValue(CACHE_NAME, key).toString();
    }

    @Override
    public Collection<String> getCacheKeys() {
        return REDIS_CACHE.getCacheMapKeys(CACHE_NAME);
    }

    @Override
    public void clearCacheByKeys(String... keys) {
        REDIS_CACHE.deleteCacheMapValue(CACHE_NAME, keys);
    }

    @Override
    public void clearCache() {
        clearCacheByKeys();
    }

    private static final String CACHE_NAME = "shipExtendCache";

    private static final RedisCache REDIS_CACHE;

    private static final SysShipExtendService sysShipExtendService;

    static {
        REDIS_CACHE = SpringUtils.getBean(RedisCache.class);
        sysShipExtendService = SpringUtils.getBean(SysShipExtendService.class);
    }


    /**
     * 清空缓存
     */
    public static void clearShipExtendCache() {
        Collection<String> keys = REDIS_CACHE.keys(CACHE_NAME);
        REDIS_CACHE.deleteObject(keys);
    }


    /**
     * 设置cache key
     *
     * @param configKey 参数键
     * @return 缓存键key
     */
    public static String getCacheKey(String configKey) {
        return CacheConstants.SYS_SHIP_EXTEND_KEY + configKey;
    }

    /**
     * 获取缓存列表
     *
     * @return
     */
    public static List<SysShipExtend> getShipExtendList() {
        List<SysShipExtend> sysShipExtends = REDIS_CACHE.getCacheObjList(CACHE_NAME);
        return sysShipExtends;
    }

    public static void setShipExtendCache(SysShipExtend sysShipExtend) {
        REDIS_CACHE.setCacheMapValue(CACHE_NAME, getCacheKey(sysShipExtend.getShipId()), sysShipExtend);
    }
}
