package com.aidex.framework.cache;

import com.aidex.common.constant.CacheConstants;
import com.aidex.common.core.redis.RedisCache;
import com.aidex.common.utils.spring.SpringUtils;
import com.aidex.system.domain.SysConfig;
import com.aidex.system.domain.SysWharf;
import com.aidex.system.service.SysWharfService;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.Collection;
import java.util.List;

/**
 * 配置工具类
 *
 * @author ruoyi
 */
@Component
public class WhrafUtils implements CacheUtil {

    @Override
    public String getCacheName() {
        return CACHE_NAME;
    }

    @Override
    public String getRemark() {
        return "站点信息";
    }

    @Override
    public String getCacheId() {
        return "whrafUtils";
    }

    @Override
    public String getCacheValue(String key) {
        return REDIS_CACHE.getCacheMapValue(CACHE_NAME, key).toString();
    }

    @Override
    public Collection<String> getCacheKeys() {
        return REDIS_CACHE.getCacheMapKeys(CACHE_NAME);
    }

    @Override
    public void clearCacheByKeys(String... keys) {
        REDIS_CACHE.deleteCacheMapValue(CACHE_NAME, keys);
    }

    @Override
    public void clearCache() {
        clearCacheByKeys();
    }

    private static final String CACHE_NAME = "wharfCache";

    private static final RedisCache REDIS_CACHE;

    private static final SysWharfService sysWharfService;

    static {
        REDIS_CACHE = SpringUtils.getBean(RedisCache.class);
        sysWharfService = SpringUtils.getBean(SysWharfService.class);
    }


    /**
     * 清空缓存
     */
    public static void clearWharfCache() {
        Collection<String> keys = REDIS_CACHE.keys(CACHE_NAME);
        REDIS_CACHE.deleteObject(keys);
    }


    /**
     * 设置cache key
     *
     * @param configKey 参数键
     * @return 缓存键key
     */
    public static String getCacheKey(String configKey) {
        return CacheConstants.SYS_WHARF_KEY + configKey;
    }

    /**
     * 获取缓存列表
     *
     * @return
     */
    public static List<SysWharf> getWharfList() {
        List<SysWharf> sysWharf = REDIS_CACHE.getCacheObjList(CACHE_NAME);
        return sysWharf;
    }

    public static void setWhrafCache(SysWharf sysWharf) {
        REDIS_CACHE.setCacheMapValue(CACHE_NAME, getCacheKey(sysWharf.getId()), sysWharf);
    }
}
