package com.aidex.system.domain;

import java.util.Date;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import com.aidex.common.core.domain.BaseEntity;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import com.aidex.common.utils.log.annotation.FieldRemark;
import com.aidex.common.utils.log.annotation.LogField;
import com.aidex.common.annotation.Excel;

/**
 * 预约配置对象 sys_reservation_config
 * @author Chensir
 * @email 914769835
 * @date 2024-03-28
 */
@Data
public class SysReservationConfig extends BaseEntity<SysReservationConfig>
{
    private static final long serialVersionUID = 1L;

    /** 预约日期 */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @Excel(name = "预约日期", width = 30, dateFormat = "yyyy-MM-dd")
    @LogField
    @FieldRemark(name = "预约日期",field = "reservationDate")
    private Date reservationDate;

    /** 预约日期开始 */
    private String beginReservationDate;

    /** 预约日期结束 */
    private String endReservationDate;

    /** 预约开始时段 */
    @JsonFormat(pattern = "HH:mm", timezone = "GMT+8")
    @Excel(name = "预约开始时段", width = 30, dateFormat = "HH:mm")
    @LogField
    @FieldRemark(name = "预约开始时段",field = "reservationStartTime")
    private Date reservationStartTime;

    /** 预约开始时段开始 */
    private String beginReservationStartTime;

    /** 预约开始时段结束 */
    private String endReservationStartTime;

    /** 预约结束时段 */
    @JsonFormat(pattern = "HH:mm", timezone = "GMT+8")
    @Excel(name = "预约结束时段", width = 30, dateFormat = "HH:mm")
    @LogField
    @FieldRemark(name = "预约结束时段",field = "reservationEndTime")
    private Date reservationEndTime;

    /** 预约结束时段开始 */
    private String beginReservationEndTime;

    /** 预约结束时段结束 */
    private String endReservationEndTime;

    /** 可预约人数 */
    @Excel(name = "可预约人数")
    @LogField
    @FieldRemark(name = "可预约人数",field = "enableNum")
    private Long enableNum;

    /** 可预约人数 */
    @Excel(name = "已预约人数")
    @LogField
    @FieldRemark(name = "已预约人数",field = "enableNum")
    private Long reservationNum;

    /** 配置状态 0 保存 1 发布 */
    @Excel(name = "配置状态", dictType = "sys_normal_disable")
    @LogField
    @FieldRemark(name = "配置状态",field = "status")
    private String status;

    public void setReservationDate(Date reservationDate)
    {
        this.reservationDate = reservationDate;
    }

    public Date getReservationDate()
    {
        return reservationDate;
    }

    public void setReservationStartTime(Date reservationStartTime)
    {
        this.reservationStartTime = reservationStartTime;
    }

    public Date getReservationStartTime()
    {
        return reservationStartTime;
    }

    public void setReservationEndTime(Date reservationEndTime)
    {
        this.reservationEndTime = reservationEndTime;
    }

    public Date getReservationEndTime()
    {
        return reservationEndTime;
    }

    public void setEnableNum(Long enableNum)
    {
        this.enableNum = enableNum;
    }

    public Long getEnableNum()
    {
        return enableNum;
    }

    public void setStatus(String status)
    {
        this.status = status;
    }

    public String getStatus()
    {
        return status;
    }

    public Long getReservationNum() {
        return reservationNum;
    }

    public void setReservationNum(Long reservationNum) {
        this.reservationNum = reservationNum;
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this,ToStringStyle.MULTI_LINE_STYLE)
            .append("reservationDate", getReservationDate())
            .append("reservationStartTime", getReservationStartTime())
            .append("reservationEndTime", getReservationEndTime())
            .append("enableNum", getEnableNum())
            .append("status", getStatus())
            .append("remark", getRemark())
            .append("id", getId())
            .append("createBy", getCreateBy())
            .append("createDept", getCreateDept())
            .append("createTime", getCreateTime())
            .append("updateBy", getUpdateBy())
            .append("updateTime", getUpdateTime())
            .append("updateIp", getUpdateIp())
            .append("version", getVersion())
            .append("delFlag", getDelFlag())
            .toString();
    }
}
