package com.aidex.system.mapper;

import com.aidex.system.domain.SysReservationData;
import com.aidex.system.domain.SysRoleMenu;
import com.aidex.system.domain.vo.SysReservationDataHistoryVO;
import com.aidex.system.domain.vo.SysReservationDataNumVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * 角色与菜单关联表 数据层
 *
 * @author ruoyi
 */
public interface SysReservationDataMapper
{
    /**
     * 根据配置获取当前预约人数
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public List<SysReservationDataNumVO> getNumByReservationConfigIds(String[] ids);

    /**
     * 新增预约数据
     *
     * @param sysReservationData 预约数据
     * @return 结果
     */
    public int addReservationData(SysReservationData sysReservationData);


    /**
     * 获取数据
     *
     * @param sysReservationData 预约数据
     * @return 结果
     */
    public List<SysReservationData> getData(SysReservationData sysReservationData);

    /**
     * 判断小程序当日有无预约
     *
     * @param sysReservationData 预约数据
     * @return 结果
     */
    public int getNumByOpenid(SysReservationData sysReservationData);

    /**
     * 判断当日有无预约
     *
     * @param date 日期
     * @return 结果
     */
    public int getNumByDate(String date);

    /**
     * 判断小程序当日有无预约
     *
     * @param sysReservationData 预约数据
     * @return 结果
     */
    public int getNumByReservationConfigId(SysReservationData sysReservationData);

    /**
     * 判断小程序当日有无预约
     *
     * @param id 预约数据id
     * @return 结果
     */
    public int deleteById(String id);

    /**
     * 获取小程序历史数据
     *
     * @param openid 小程序用户名
     *
     * @return 结果
     */
    public List<SysReservationDataHistoryVO> getHistoryData(@Param("openid") String openid, @Param("type") String type, @Param("date") String date);

    /**
     * 根据日期获取预约时间点
     *
     * @param date 预约数据
     * @return 结果
     */
    public List<String> getDurationByDate(String date);

    /**
     * 根据日期获取预约时间点
     *
     * @param date 预约数据
     * @return 结果
     */
    public List<Map<String, String>> getReservationDataByDate(String date);


}
