package com.aidex.system.service;

import com.aidex.common.core.service.BaseService;
import com.aidex.system.domain.SysShip;
import com.aidex.system.domain.SysShipCamera;

import java.util.List;

/**
 * 船只管理Service接口
 * @author ChenSir
 * @email 91476835
 * @date 2024-03-16
 */
public interface SysShipService extends BaseService<SysShip> {

    /**
     * 批量删除船只管理
     * @param ids 需要删除的船只管理ID集合
     * @return 结果
     */
    public int deleteSysShipByIds(String[] ids);


    /**
     * @param macId 船只编号
     * @return 结果
     */
    public SysShip getShipByMacId(String macId);

    /**
     * 更新船只状态
     * @param macId 船只编号
     * @return 结果
     */
    public int updateStatusByMacId(String macId, String status);

    /**
     * 按状态查询在线船只数量-包含在线静止与在线运行
     * @return 结果
     */
    public int countSysShipByStatus();

    /**
     * 绑定船只与摄像头
     * @param sysShipCamera
     * @return
     */
    public int bindCamera(SysShipCamera sysShipCamera);

    /**
     * 解绑船只与摄像头
     * @param sysShipCamera
     * @return
     */
    public int unbindCamera(SysShipCamera sysShipCamera);

    /**
     * 查询船只与摄像头关联信息
     * @param shipId 船只编号
     * @return 结果
     */
    public List<SysShipCamera> getShipCameraList(String shipId);

    /**
     * 获取剩余座位数
     * @param shipId 船只编号
     * @param duration 持续时间-秒
     * @return 结果
     */
    public String getShipSurplusSeat(String shipId, int duration);



}
