package com.aidex.system.service.impl;

import com.aidex.common.core.domain.BaseEntity;
import com.aidex.common.core.page.PageDomain;
import com.aidex.common.utils.StringUtils;
import com.aidex.common.utils.sql.SqlUtil;
import com.aidex.system.domain.SysAlarmPlush;
import com.aidex.system.mapper.SysAlarmPlushMapper;
import com.aidex.system.service.GpsAlarmPlushService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 报警信息Service业务层处理
 * @author ChenSir
 * @email 914769835
 * @date 2024-03-17
 */
@Service
public class GpsAlarmPlushImpl  implements GpsAlarmPlushService {

    private static final Logger log = LoggerFactory.getLogger(GpsAlarmPlushImpl.class);

    @Autowired
    private SysAlarmPlushMapper gistPlushMapper;

    @Override
    public PageInfo<SysAlarmPlush> findPages(SysAlarmPlush sysAlarmPlush) {
        PageDomain page = sysAlarmPlush.getPage();
        Integer pageNum = page.getPageNum();
        Integer pageSize = page.getPageSize();
        if (StringUtils.isNotNull(pageNum) && StringUtils.isNotNull(pageSize))
        {
            String orderBy = SqlUtil.escapeOrderBySql(page.getOrderBy());
            Boolean reasonable = page.getReasonable();
            PageHelper.startPage(pageNum, pageSize, orderBy).setReasonable(reasonable);
        }
        return new PageInfo(gistPlushMapper.list(sysAlarmPlush));
    }

    @Override
    public int updateStatus(SysAlarmPlush sysAlarmPlush) {
        sysAlarmPlush.setStatus(BaseEntity.STATUS_NORMAL.equals(sysAlarmPlush.getStatus()) ? BaseEntity.STATUS_DISABLE : BaseEntity.STATUS_NORMAL);
        return gistPlushMapper.updateStatus(sysAlarmPlush);
    }

    @Override
    public int getCount() {
        return gistPlushMapper.getNumByStatus(BaseEntity.STATUS_NORMAL);
    }

    @Override
    public List<SysAlarmPlush> findList(SysAlarmPlush sysAlarmPlush) {
        return gistPlushMapper.list(sysAlarmPlush);
    }
}
