package com.aidex.system.service.impl;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.http.HttpRequest;
import com.aidex.common.core.domain.BaseEntity;
import com.aidex.common.core.service.BaseServiceImpl;
import com.aidex.common.plush.domain.SysShipExtend;
import com.aidex.common.plush.service.SysShipExtendService;
import com.aidex.common.utils.SecurityUtils;
import com.aidex.common.utils.StringUtils;
import com.aidex.system.domain.SysShip;
import com.aidex.system.domain.SysShipCamera;
import com.aidex.system.domain.vo.ShipVo;
import com.aidex.system.mapper.SysShipMapper;
import com.aidex.system.service.SysShipService;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.github.pagehelper.PageInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 船只管理Service业务层处理
 * @author ChenSir
 * @email 91476835
 * @date 2024-03-16
 */
@Service
@Transactional(readOnly = true)
public class SysShipServiceImpl extends BaseServiceImpl<SysShipMapper, SysShip> implements SysShipService {

    private static final Logger log = LoggerFactory.getLogger(SysShipServiceImpl.class);

    @Autowired
    private SysShipExtendService sysShipExtendService;


    /**
     * 获取单条数据
     * @param sysShip 船只管理
     * @return 船只管理
     */
    @Override
    public SysShip get(SysShip sysShip) {
        SysShip dto = super.get(sysShip);
        return dto;
    }

    /**
     * 获取单条数据
     * @param id 船只管理id
     * @return 船只管理
     */
    @Override
    public SysShip get(String id) {
        SysShip dto = super.get(id);
        return dto;
    }

    /**
     * 查询船只管理列表
     * @param sysShip 船只管理
     * @return 船只管理
     */
    @Override
    public List<SysShip> findList(SysShip sysShip) {
		List<SysShip> sysShipList = super.findList(sysShip);
        return sysShipList;
    }

    /**
     * 分页查询船只管理列表
     * @param sysShip 船只管理
     * @return 船只管理
     */
    @Override
    public PageInfo<SysShip> findPage(SysShip sysShip) {
		PageInfo<SysShip> page = super.findPage(sysShip);
        return page;
    }

    /**
     * 保存船只管理
     * @param sysShip
     * @return 结果
     */
    @Override
    public boolean save(SysShip sysShip) {
        ShipVo shipVo = new ShipVo();
        BeanUtils.copyProperties(sysShip,shipVo);
        SysShip ship = new SysShip();
        BeanUtils.copyProperties(sysShip,ship);
        if(StringUtils.isEmpty(ship.getRemark()))
            ship.setRemark("");
        if(super.save(ship)) {
            SysShipExtend shipExtend = new SysShipExtend();
            shipExtend.setShipId(sysShip.getId());
            shipExtend.setId(sysShip.getId());
            shipExtend.setMacId(sysShip.getShipNum());
            shipExtend.setDir((StringUtils.isEmpty(shipVo.getTravel()) || shipVo.getTravel().equals("1"))?"100":"270");
            shipExtend.setIsStop(1L);
            shipExtend.setLat(!StringUtils.isEmpty(sysShip.getLat())?sysShip.getLat():"36.077395");
            shipExtend.setLon(!StringUtils.isEmpty(sysShip.getLng())?sysShip.getLng():"103.863987");
            shipExtend.setMileage("0");
            shipExtend.setDelFlag("0");
            shipExtend.setWharfStop(!StringUtils.isEmpty(shipVo.getWharf())?shipVo.getWharf():"f502c163626e4aefa320bb7d5f47824c");
            shipExtend.setTravel(1L);
            shipExtend.setSpeed("0");
            if (sysShip.getIsNewRecord()) {
                sysShipExtendService.save(shipExtend);
            }else {
                if(sysShipExtendService.findById(sysShip.getId()) == null){
                    sysShipExtendService.save(shipExtend);
                }else {
                    sysShipExtendService.updateByShip(shipExtend);
                }
            }
            return Boolean.TRUE;
        }else
            return Boolean.FALSE;
    }

    /**
     * 删除船只管理信息
     * @param sysShip
     * @return 结果
     */
    @Override
    public boolean remove(SysShip sysShip) {
        if(super.remove(sysShip)) {
            sysShipExtendService.delByIds(sysShip.getId().split(","));
            return Boolean.TRUE;
        }else
            return Boolean.FALSE;
    }

    /**
     * 批量删除船只管理
     * @param ids 需要删除的船只管理ID
     * @return 结果
     */
    @Transactional(readOnly = false)
    @Override
    public int deleteSysShipByIds(String[] ids) {
        sysShipExtendService.delByIds(ids);
        return mapper.deleteSysShipByIds(ids, BaseEntity.DEL_FLAG_DELETE);
    }

    @Override
    public SysShip getShipByMacId(String macId) {
        return mapper.getShipByMacId(macId, BaseEntity.DEL_FLAG_NORMAL);
    }

    @Transactional(readOnly = false)
    @Override
    public int updateStatusByMacId(String macId, String status) {
        return mapper.updateStatusByMacId(macId,status);
    }

	@Override
	public int countSysShipByStatus() {
		return mapper.countSysShipByStatus();
	}
    @Transactional(readOnly = false)
    @Override
    public int bindCamera(SysShipCamera sysShipCamera) {
        List<SysShipCamera> shipCameraList = getShipCameraList(sysShipCamera.getShipId());
        if (shipCameraList.size() >= 2) {
            return 0;
        }
        return mapper.bindCamera(sysShipCamera);
    }
    @Transactional(readOnly = false)
    @Override
    public int unbindCamera(SysShipCamera sysShipCamera) {
        return mapper.unbindCamera(sysShipCamera);
    }

    @Override
    public List<SysShipCamera> getShipCameraList(String shipId) {
        return mapper.getShipCameraList(shipId);
    }

    @Override
    public String getShipSurplusSeat(String shipId, int duration) {
        // 获取船只关联的摄像头
        List<SysShipCamera> shipCameraList = getShipCameraList(shipId);
        if (shipCameraList.size() > 0){
            String redisKey = IdUtil.fastSimpleUUID();
            DateTime offset = DateUtil.offset(new Date(), DateField.SECOND, duration);
            String corn = DateUtil.format(offset, "ss mm HH dd MM ? yyyy");
            log.info("生成corn 表达式为---{}", corn);
            // 构造带参定时任务
            String name = "定时获取船只剩余座位-" + redisKey;
            Map<String, String> params = new HashMap<>();
            params.put("jobName",  name);
            params.put("jobGroup", "DEFAULT");
            params.put("invokeTarget", "cameraTask.getShipSurplusSeat('"+shipId+"','"+DateUtil.now()+"','"+redisKey+"')");
            params.put("cronExpression",corn);
            params.put("misfirePolicy","2");
            params.put("concurrent","0");
            params.put("status","0");
            //链式构建请求
            String result = HttpRequest.post("http://127.0.0.1:18080/monitor/job")
                    .body(JSON.toJSONString(params))//表单内容
                    .timeout(10000)//超时，毫秒
                    .execute().body();
            JSONObject resultJson = JSONObject.parseObject(result);
            if (resultJson.containsKey("code") && resultJson.get("code").equals(200)){
                String job = HttpRequest.get("http://127.0.0.1:18080/monitor/job/byName/"+ name)
                        .timeout(10000)//超时，毫秒
                        .execute().body();
                JSONObject jobJson = JSONObject.parseObject(job);
                if (jobJson.containsKey("code") && jobJson.get("code").equals(200)){
                    JSONObject jobData = jobJson.getJSONObject("data");
                    jobData.put("status", "0");
                    if (jobData.containsKey("jobId")){
                        // 修改任务状态
                        HttpRequest.put("http://127.0.0.1:18080/monitor/job/changeStatus")
                                .body(JSON.toJSONString(jobData))//表单内容
                                .timeout(10000)//超时，毫秒
                                .execute();
                    }
                }
                return redisKey;
            }
		}
        return null;
    }

}
